// Download WebR runtime
const { WebR, ChannelType } = await import('https://webr.r-wasm.org/latest/webr.mjs');

// Initialise WebR
let webR;
try {
	webR = new WebR({
		channelType: ChannelType.PostMessage,
	});
	await webR.init();
} catch(e) {
	error(e.message);
}

// Set default render device
await webR.evalRVoid('options(device=webr::canvas)');

// Load the R code
const code = await codea.recv();
await webR.evalR(code);

// Restructures RObject js results into Codea friendly structures
// for transfer back to Codea.
const transferableResult = (r)=>{
	if (typeof r == "object") {
		if (r.type === "null") {
			return null;
		}
		
		if (r.names != null) {
			let o = new Object();
			let i = 0;
			for (const key of r.names) {
				o[key] = transferableResult(r.values[i++]);
			}
			return o;
		} else {
			let a = new Array();
			for (const val of r.values) {
				a.push(transferableResult(val));
			}
			if (a.length == 1) {
				return a[0];
			}
			return a;
		}
	}
	return r;
};

// Our plot canvas
let gfxReceiverId = 1;
let canvas = new OffscreenCanvas(1008, 1008);
let isNewCanvas = false;

// Async message handler
(async ()=>{
	while (true) {
		const output = await webR.read();
		switch(output.type) {
			case "stdout":
				//print(output.data);
				break;
			case "stderr":
				print(output.data);
				break;
			case "prompt":
				//print(">");
				break;
			case "canvas":
				if (output.data.event === 'canvasImage') {
					const width = output.data.image.width;
					const height = output.data.image.height;
					
					// Is the canvas the correct size
					// or should we create a new one anyway?
					if (canvas.width != width ||
						canvas.height != height || isNewCanvas == true)
					{
						canvas = new OffscreenCanvas(width, height);
						isNewCanvas = true;
					}
					
					// Draw to our canvas
					canvas.getContext('2d').drawImage(output.data.image, 0, 0);
					
					// Prepare and send our updated image
					let data = await canvas.convertToBlob();
					data = await data.arrayBuffer();
					data = new Uint8Array(data);
					
					codea.send({
						data: {
							data: data,
							isNewPlot: isNewCanvas
						},
						id: gfxReceiverId
					});
					isNewCanvas = false;
				
				} else if (output.data.event === 'canvasNewPage') {
					isNewCanvas = true;
				}
				break;
			default:
				print("R message:", output.type);
				break;
		}
	}
})();

// Signal to Codea that the instance is ready
codea.send("_ready_");